# medz/plugs/__init__.py
#
#

""" medz plugins. """

__copyright__ = "Copyright 2015, B.H.J Thate"

## IMPORTS

from medz.dispatcher import Dispatcher
from medz.errors import error
from medz import Thing

import importlib
import logging
import pkgutil

## PLUGINS

class Plugs(Dispatcher):

    """ register plugins. """

    ## INIT

    def init(zelf, plugs):
        """ initialize plugins, """
        modnames = plugs.split(",") 
        for modname in sorted(modnames, reverse=True):
            if not modname: continue
            objs = zelf.find(modname)
            for obj in objs:
                logging.warn("# init %s" % modname)
                if modname == "xmpp": obj.init() ; continue
                init = getattr(obj, "init", None)
                if init: obj.init()
        zelf.ready()

    def boot(zelf, *args, **kwargs):
        from medz.runtime import kernel
        zelf.init(",".join(kernel.keys()))

    def stop(zelf, plugs):
        """ stop list of plugins, """
        modnames = plugs.split(",") 
        for name in sorted(modnames)[::-1]:
            modnames = zelf.find(name)
            for name in modnames:
                obj = zelf.get(name)
                if not obj: continue
                try: obj.stop()
                except: pass

    ## SEARCH

    def search(zelf, *args, **kwargs):
        """ find matching name. """
        n = args[0]
        for name in zelf.names():
            if n in name: yield name

    ## LOAD

    def load(zelf, modname):
        """ load plugins from the plugin directory. """
        logging.warn("# plugs %s" % modname)
        mod = zelf.loading(modname)
        for pkg in pkgutil.walk_packages(mod.__path__, mod.__name__+'.'):
            name = pkg[1]
            zelf[name] = zelf.loading(name, modname)

    def loading(zelf, name, package=None, force=True):
        """ load module. """
        logging.info("! mod %s" % ".".join(name.split(".")[1:]))
        if name in zelf and not force: mod = zelf.reload(name)
        else: mod = importlib.import_module(name, package)
        return mod

    ## RELOAD

    def reload(zelf, *args, **kwargs):
        name = args[0]
        logging.warn("# reload %s" % name)
        zelf[name] = importlib.reload(zelf[name])
        return zelf[name]

    def reloading(zelf, *args, **kwargs):
        names = zelf.find(args[0])
        for name in names: zelf.reload(name)
